@@ -1,4 +1,4 @@
-## Copyright (c) 2004-2012, Ph. Grosjean <phgrosjean@sciviews.org>
+## Copyright (c) 2004-2015, Ph. Grosjean <phgrosjean@sciviews.org>
 ##
 ## This file is part of ZooImage
 ## 
@@ -160,7 +160,7 @@ template = c("[Basic]", "[Detailed]", "[Very detailed]"), classes = NULL, ...)
             ## Link .zidb database to R objects in memory
             Zidb <- zidbLink(zidbfiles[i])
             AllItems <- ls(Zidb)
-            Vigns <- AllItems[-grep("_dat1", AllItems)]
+            Vigns <- AllItems[!grepl("_dat1", AllItems)]
             ## Extract all vignettes in their class subdirectory
             imgext <- Zidb[[".ImageType"]]
 			## Get path for the vignettes and copy them there
@@ -244,7 +244,7 @@ addToTrain <- function (traindir, zidbfiles, classes = NULL, ...)
 			## Link .zidb database to R objects in memory
             Zidb <- zidbLink(zidbfile)
             AllItems <- ls(Zidb)
-            Vigns <- AllItems[-grep("_dat1", AllItems)]
+            Vigns <- AllItems[!grepl("_dat1", AllItems)]
             ## Copy all vignettes in the TopPath directory
             imgext <- Zidb[[".ImageType"]]
 			## Get path for the vignettes and copy them there
@@ -380,6 +380,8 @@ na.rm = FALSE)
 
 	## Rename Dat in df
 	df <- Dat
+	## Fix ECD in case of FIT_VIS data
+	if ("FIT_Area_ABD" %in% names(df)) df$ECD <- ecd(df$FIT_Area_ABD)
 	## Problem if there is no remaining row in the data frame
 	if (nrow(df) == 0) {
 		warning("No valid item found (no vignettes with valid measurement data)")
@@ -495,15 +497,22 @@ recode.ZITrain <- function (object, new.levels, depth, ...)
 		if (!missing(new.levels))
 			warning("depth is provided, so, new.levels is ignored and recomputed")
 		new.levels <- .recodeLevels(object, depth)
+		levels <- basename(attr(object, "path"))
+		## Check that levels match those defined in the Class variable
+		levels2 <- levels(object$Class)
+		if (length(levels) != length(levels2))
+			stop("length of levels in 'path' attribute must match levels in object$Class")
+		if (any(!levels %in% levels2))
+			stop("levels in the 'path' attribute do not match levels in object$Class")
+	} else {
+		## If new.levels is provided, just assume they match levels(object$Class)
+		levels <- levels(object$Class)
+		if (length(new.levels) != length(levels))
+			stop("length of new.levels must match levels in object$Class")
 	}
-	
-	## Check that new.levels is of the same length as levels(object$Class)
-	## [and object$Predicted or Predicted2, possibly]
-	levels <- levels(object$Class)
 	new.levels <- as.character(new.levels)
-	if (length(new.levels) != length(levels))
-		stop("length of new.levels must match levels in object$Class")
 	
+	## Change levels now
 	relevel <- function (x, levels, new.levels) {
 		x <- as.character(x)
 		for (i in 1:length(levels))
